/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
#include "system.h"
#include "system_config.h"

#include "app_device_cdc_basic.h"
//#include "app_led_usb_status.h"

#include "usb.h"
#include "usb_device.h"
#include "usb_device_cdc.h"

void USB_TASK();
void UARTreadTEXT_1();
void UARTreadTEXT_2();
void MyWait(int iT);

char gc[1];

/********************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *******************************************************************/
MAIN_RETURN main(void)
{
    ADCON1  =	0b00001111;
    TRISA   =	0b00000000;
    TRISB   =	0b00000000;
    TRISC   =	0b00110000;         //D-,D+
    TRISD   =	0b00000000;
    TRISE   =   0b11000011;         //SW1,2=INPUT LED=OUTPUT
    LATA    =	0b00000000;
    LATB    =	0b00000000;
    LATC    =	0b00000000;
    LATD    =	0b00000000;
    LATE    =	0b00000000;

    SYSTEM_Initialize(SYSTEM_STATE_USB_START);
    USBDeviceInit();
    USBDeviceAttach();

    UART_Init(115200);
    RCIF = 0; //reset RX pin flag
    RCIP = 1; //high priority
    RCIE = 1; //Enable RX interrupt
    PEIE = 1; //Enable pheripheral interrupt (serial port is a pheripheral)
    INTCONbits.GIE = 1;

    int i;
    LED=0;
    while(SW1==1)
    {
        UARTreadTEXT_1();
        USB_TASK();
    }
    UART_Write_Text("AT+CWMODE=1\r\n");
    MyWait(100);
    UARTreadTEXT_1();
    UART_Write_Text("AT+CWJAP=\"SSID\",\"password\"\r\n");
    MyWait(800);
    UARTreadTEXT_1();
    //UART_Write_Text("AT+CIFSR\r\n");
    UART_Write_Text("AT+CIPSTART=\"TCP\",\"パソコンIPアドレス\",ポート\r\n");
    MyWait(200);
    UARTreadTEXT_1();
    UART_Write_Text("AT+CIPMODE=1\r\n");
   MyWait(200);
    UARTreadTEXT_1();
    UART_Write_Text("AT+CIPSEND\r\n");
   MyWait(200);
    UARTreadTEXT_1();

    UART_Write_Text(">OK\r\n");
    gN=0;
    __delay_ms(15);
    while(1)
    {
        LED=1;
        UARTreadTEXT_2();
        USB_TASK();
    }//end while
}//end main

void USB_TASK()
{
    SYSTEM_Tasks();
    #if defined(USB_POLLING)
    #endif
    if( USBGetDeviceState() < CONFIGURED_STATE )
    {
        return;
    }
    if( USBIsDeviceSuspended()== true )
    {
        return;
    }
    MyCDCBasicTasks();
}


void MyWait(int iT)
{
    int i,ii;
    for(ii=1;ii<iT;ii++)
    {
        __delay_ms(10);
        if(gN>=1)
        {
            for(i=1;i<gN;i++)
            {
                if(UARTreadBuffer[i-1]=='O' && UARTreadBuffer[i]=='K')
                {
                    __delay_ms(5);
                    return;
                }

            }
        }
    }
}


void UARTreadTEXT_1()
{
    int i;
    uint8_t Buffer[256];
    if(gN>=1)
    {
        while(USBUSARTIsTxTrfReady() == false){CDCTxService();}
        uint8_t iN=gN;
        gN=0;
        for(i=0;i<iN;i++)
        {
            Buffer[i]=UARTreadBuffer[i];
        }
        putUSBUSART(Buffer,iN);//USB送信
    }
    CDCTxService();
}
void UARTreadTEXT_2()
{
    int i;
    uint8_t Buffer[256];
    if(gN>=1)
    {
       while(USBUSARTIsTxTrfReady() == false){CDCTxService();}
       uint8_t iN=gN;
       gN=0;
       for(i=0;i<iN;i++)
       {
           Buffer[i]=UARTreadBuffer[i];
       }
       Buffer[iN]=0x00;
       putUSBUSART(Buffer,iN);//USB送信
       UART_Write_Text(Buffer);
    }
    CDCTxService();
}




bool USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, uint16_t size)
{
    switch( (int) event )
    {
        case EVENT_TRANSFER:
            break;

        case EVENT_SOF:
            /* We are using the SOF as a timer to time the LED indicator.  Call
             * the LED update function here. */
            break;

        case EVENT_SUSPEND:
            /* Update the LED status for the suspend event. */
            break;

        case EVENT_RESUME:
            /* Update the LED status for the resume event. */
            break;

        case EVENT_CONFIGURED:
            /* When the device is configured, we can (re)initialize the 
             * demo code. */
            APP_DeviceCDCBasicDemoInitialize();
            break;

        case EVENT_SET_DESCRIPTOR:
            break;

        case EVENT_EP0_REQUEST:
            /* We have received a non-standard USB request.  The HID driver
             * needs to check to see if the request was for it. */
            USBCheckCDCRequest();
            break;

        case EVENT_BUS_ERROR:
            break;

        case EVENT_TRANSFER_TERMINATED:
            break;

        default:
            break;
    }
    return true;
}

/*******************************************************************************
 End of File
*/
